package creim.art.artist;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import creim.gui.ImageFrame;
import java.awt.AlphaComposite;



public abstract class Artist {
	
	protected int width;
	protected int height;
	protected BufferedImage image;
	
	public Artist(int width, int height) {
		this.width = width;
		this.height = height;
		this.image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
	}
	
	
	public abstract BufferedImage paint();
	
	
	public BufferedImage getImage() {
		return image;
	}
	
	public void clear() {
		image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
	}
	
	public void append(BufferedImage layer) {
		append(layer, 0, 0);
	}
	
	public void append(BufferedImage layer, int x, int y) {
		Graphics2D g = (Graphics2D)image.getGraphics();
		g.drawImage(layer, x, y, null);
		g.dispose();
	}
	
	public void append(BufferedImage layer, int x, int y, float alpha) {
		Graphics2D g = (Graphics2D)image.getGraphics();
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		g.drawImage(layer, x, y, null);
		g.dispose();
	}
	
	public void store(File outputfile) throws IOException {
		if (!outputfile.getParentFile().exists())
			outputfile.getParentFile().mkdirs();
		if (!outputfile.toString().contains(".png"))
			outputfile = new File(outputfile.getParentFile(), outputfile.getName().concat(".png"));
		ImageIO.write(image, "PNG", outputfile);
	}
	
	public void display() {
		ImageFrame frame = new ImageFrame(width, height, image);
		frame.setVisible(true);
	}
	
	public void display(Color bg) {
		ImageFrame frame = new ImageFrame(width, height, image, bg);
		frame.setVisible(true);
	}
}
