package creim.art.artist.description;

import java.awt.Color;
import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.analysis.audio.AudioAnalysisResult;
import creim.art.artist.Artist;
import creim.graphics.colors.RYBColorWheel;
import creim.graphics.effect.BlurEffect;
import creim.graphics.effect.OilifyEffect;
import creim.art.painter.StraightBrushPainter;
import creim.util.Logger;



public class BrushArtist extends Artist {

	private AudioAnalysisResult features;
	private Color[] palette;
	private Color bgcolor;
	private int strokes;
	private int rotation;
	private double RYBdeg;
	
	public BrushArtist(int width, int height, AudioAnalysisResult features) {
		super(width, height);
		this.features = features;
		init();
	}
	
	
	private void init() {
		double energy = features.spotifyanalysis.energy;
		double loud = features.spotifyanalysis.loudness;
		double danceability = features.spotifyanalysis.danceability;
		double valence = features.spotifyanalysis.valence;
		double tempo = features.spotifyanalysis.tempo;
		
//		strokes = (int)MathUtil.scale(energy, 0, 1, 1, 20);
		strokes = (int)(tempo * 0.5);
//		d.boxesCountDom = new Domain(boxes, boxes);
//		d.boxesMinDim = new Dimension((int)MathUtil.scale(energy, 0, 1, 1, 20), (int)MathUtil.scale(energy, 0, 1, 1, 20));
//		d.boxesMaxDim = new Dimension((int)MathUtil.scale(energy, 0, 1, 20, Math.max(imagedim.width, imagedim.height)), (int)MathUtil.scale(energy, 0, 1, 20, Math.max(imagedim.width, imagedim.height)));
	
		
		// danceability -> angles on strokes
		if (danceability > 0.5)
			rotation = (int)MathUtil.scale(danceability, 0, 1, 0, 360);
		else 
			rotation = 0;
		
		RYBdeg = RYBColorWheel.heat(energy);
		palette = RYBColorWheel.adjacent(RYBdeg, valence*90);
		
		bgcolor = Color.WHITE;
//		if (features.spotifyfeatures.acousticness > 0.75) {
//			palette = RYBColorWheel.adjacent(180, 15);
//			bgcolor = Color.GRAY;
//		}
		
	}

	
	@Override
	public BufferedImage paint() {
//		ClearPainter clearcoat = new ClearPainter(width, height, palette[0]);
//		clearcoat.paintOn(canvas);
		
		Logger.info("Background ...");
		StraightBrushPainter bg = new StraightBrushPainter(0, 0, width, height+100, bgcolor, 0);
		bg.paintOn(image);
		
		Logger.info("Painting ...");
		for (int i = 0; i < strokes; i++) {
			int x = RandomUtil.nextInt(-100, width);
			int y = RandomUtil.nextInt(-100, height);
			int w = RandomUtil.nextInt(5, 500);
			int h = RandomUtil.nextInt(5, 500);
			
			StraightBrushPainter p = new StraightBrushPainter(x, y, w, h, palette[RandomUtil.nextInt(palette.length)], RandomUtil.nextInt(0, rotation));
			p.paintOn(image);
		}
		
		Logger.info("Oil ...");
		OilifyEffect oil = new OilifyEffect(image, 5, 15);
		image = oil.apply();
		
		Logger.info("Blur ...");
		BlurEffect blur = new BlurEffect(image, 1);
		image = blur.apply();
		
		return image;
	}

}
