package creim.art.artist.description;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.CubicCurve2D;
import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.analysis.audio.AudioAnalysisResult;
import creim.art.artist.Artist;
import creim.graphics.colors.ColorPalette;
import creim.graphics.colors.RYBColorWheel;
import creim.graphics.effect.BlurEffect;
import creim.graphics.effect.OilifyEffect;
import creim.art.painter.CurveBrushPainter;
import creim.util.Logger;
import org.apache.commons.math3.stat.StatUtils;



public class BrushZoneLineArtist extends Artist {
	
	private AudioAnalysisResult features;
	private Color[] palette;
	private Color bgcolor;
	private int strokes;
	private int rotation;
	private double RYBdeg;
	private double[][] zones;
	private double zoneglobalmax;
	private double zoneglobalmin;
	

	public BrushZoneLineArtist(int width, int height, AudioAnalysisResult features) {
		super(width, height);
		this.features = features;
		init();
	}
	
	
	private void init() {
		zones = features.zones;
		for (double[] zone : zones) {
			zoneglobalmax = Math.max(zoneglobalmax, StatUtils.max(zone));
			zoneglobalmin = Math.max(zoneglobalmin, StatUtils.min(zone));
		}
			
		double energy = features.spotifyanalysis.energy;
		double loud = features.spotifyanalysis.loudness;
		
		
		strokes = (int)(features.spotifyanalysis.tempo * 0.75);
		
		palette = ColorPalette.generateRandomRYBAdjacentPalette();
		
		// danceability -> angles on strokes
		double danceability = features.spotifyanalysis.danceability;
		if (danceability > 0.5)
			rotation = (int)MathUtil.scale(danceability, 0, 1, 0, 360);
		else 
			rotation = 0;
		
		double valence = features.spotifyanalysis.valence;
		RYBdeg = RYBColorWheel.heat(energy);
		palette = RYBColorWheel.adjacent(RYBdeg, valence*90);
		
		bgcolor = Color.WHITE;
	}
	
	
	@Override
	public BufferedImage paint() {
		Graphics2D g2 = (Graphics2D)image.getGraphics();
		
		Logger.info("Brush strokes ...");
		for (int i = 0; i < strokes; i++) {
			double x1 = RandomUtil.nextInt(width);
			double y1 = RandomUtil.nextInt(height);
			double x2 = RandomUtil.nextInt(width);
			double y2 = RandomUtil.nextInt(height);
			double ctrlx1 = RandomUtil.nextDouble(Math.min(x1, x2), Math.max(x1, x2));
			double ctrly1 = RandomUtil.nextDouble(Math.min(y1, y2), Math.max(y1, y2));
			double ctrlx2 = RandomUtil.nextDouble(Math.min(x1, x2), Math.max(x1, x2));
			double ctrly2 = RandomUtil.nextDouble(Math.min(y1, y2), Math.max(y1, y2));

			CurveBrushPainter p = new CurveBrushPainter(25, palette[RandomUtil.nextInt(palette.length)]);
			Shape s = new CubicCurve2D.Double(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
			p.addShape(s);
			p.paintOn(image);
		}
		
		int zonewidth = width / zones.length; // across time
		int zoneheight = height / zones[0].length; // across freqs
		
//		
//		for (int t = 0; t < zones.length; t++) { // across time
//			double[] z = zones[t]; // across freq
//			
//			double max = StatUtils.max(z);
//			double min = StatUtils.min(z);
//			
//			for (int f = 0; f < z.length; f++) {
//				
//				int zonestrokes = (int)MathUtil.scale(z[f], min, max, 0, 5);
//			
//				for (int i = 0; i < zonestrokes; i++) {
//					int colorindex = (int)MathUtil.scale(z[f % z.length], min, max, 0, palette.length-1);
//
////					int x = RandomUtil.nextInt(t * zonewidth, (t+1) * zonewidth);
////					int y = RandomUtil.nextInt(height - (f+1) * zoneheight, height - f * zoneheight);
//					int x = (t * zonewidth) + (zonewidth / 2);
//					int y = (height - ((f + 1) * zoneheight + zoneheight / 2));
//					int w = RandomUtil.nextInt(5, zonewidth) * 3;
//					int h = RandomUtil.nextInt(5, zoneheight) * 3;
//					
//					double x1 = x;
//					double y1 = y;
//					double x2 = x + w * RandomUtil.rand_neg1_to_pos1();
//					double y2 = y + h * RandomUtil.rand_neg1_to_pos1();
//					double ctrlx1 = RandomUtil.nextInt(Math.min(x, x+w), Math.max(x, x+w));
//					double ctrly1 = RandomUtil.nextInt(Math.min(y, y+h), Math.max(y, y+h));
//					double ctrlx2 = RandomUtil.nextInt(Math.min(x, x+w), Math.max(x, x+w));
//					double ctrly2 = RandomUtil.nextInt(Math.min(y, y+h), Math.max(y, y+h));
//					
//					BrushCurvePainter p = new BrushCurvePainter((int)(z[f] * 2.0));
//					Shape s = new CubicCurve2D.Double(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
//					p.addShape(s);
//					p.setColor(ColorUtil.heatmap(z[f], min, max));
//					p.paintOn(image);
//
////					StraightBrushPainter p  = new StraightBrushPainter(x, y, w, h, RandomUtil.nextInt(0, 360));
////					p.setColor(ColorUtil.heatmap(z[f], min, max));
////					p.paintOn(image);
//				}
//			}
//		}


		Logger.info("Oil ...");
		OilifyEffect oil = new OilifyEffect(image, 5, 15);
		image = oil.apply();
		
		Logger.info("Blur ...");
		BlurEffect blur = new BlurEffect(image, 1);
		image = blur.apply();
		
		return image;
	}
}
