package creim.art.artist.description;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.analysis.audio.AudioAnalysisResult;
import creim.art.artist.Artist;
import creim.graphics.colors.ColorUtil;
import creim.graphics.colors.ColorPalette;
import creim.graphics.colors.RYBColorWheel;
import creim.graphics.effect.BlurEffect;
import creim.graphics.effect.OilifyEffect;
import creim.art.painter.StraightBrushPainter;
import creim.util.Logger;
import org.apache.commons.math3.stat.StatUtils;



public class BrushZoneSpectrumArtist extends Artist {

	private AudioAnalysisResult features;
	private Color[] palette;
	private Color bgcolor;
	private int strokes;
	private int rotation;
	private double RYBdeg;
	private double[][] zones;
	private double zoneglobalmax;
	private double zoneglobalmin;
	
	
	public BrushZoneSpectrumArtist(int width, int height, AudioAnalysisResult features) {
		super(width, height);
		this.features = features;
		init();
	}
	
	
	private void init() {
		zones = features.zones;
		for (double[] zone : zones) {
			zoneglobalmax = Math.max(zoneglobalmax, StatUtils.max(zone));
			zoneglobalmin = Math.max(zoneglobalmin, StatUtils.min(zone));
		}
			
		double energy = features.spotifyanalysis.energy;
		double loud = features.spotifyanalysis.loudness;
		
//		strokes = (int)MathUtil.scale(energy, 0, 1, 1, 20);
		strokes = (int)(features.spotifyanalysis.tempo * 0.5);
//		d.boxesCountDom = new Domain(boxes, boxes);
//		d.boxesMinDim = new Dimension((int)MathUtil.scale(energy, 0, 1, 1, 20), (int)MathUtil.scale(energy, 0, 1, 1, 20));
//		d.boxesMaxDim = new Dimension((int)MathUtil.scale(energy, 0, 1, 20, Math.max(imagedim.width, imagedim.height)), (int)MathUtil.scale(energy, 0, 1, 20, Math.max(imagedim.width, imagedim.height)));
		
		palette = ColorPalette.generateRandomRYBAdjacentPalette();
		
		// danceability -> angles on strokes
		double danceability = features.spotifyanalysis.danceability;
		if (danceability > 0.5)
			rotation = (int)MathUtil.scale(danceability, 0, 1, 0, 360);
		else 
			rotation = 0;
		
		double valence = features.spotifyanalysis.valence;
		RYBdeg = RYBColorWheel.heat(energy);
		palette = RYBColorWheel.adjacent(RYBdeg, valence*90);
		
		bgcolor = Color.WHITE;
	}

	
	@Override
	public BufferedImage paint() {
		Graphics2D g2 = (Graphics2D)image.getGraphics();
		
//		ClearPainter clearcoat = new ClearPainter(width, height, palette[0]);
//		clearcoat.paintOn(canvas);
		
		Logger.info("Background ...");
//		StraightBrushPainter bg = new StraightBrushPainter(0, 0, width, height*2);
//		bg.setColor(bgcolor);
//		bg.paintOn(image);
		
		Logger.info("Painting ...");
		int zonewidth = width / zones.length; // across time
		int zoneheight = height / zones[0].length; // across freqs
		
		
		for (int t = 0; t < zones.length; t++) { // across time
			double[] z = zones[t]; // across freq
			
			double max = StatUtils.max(z);
			double min = StatUtils.min(z);
			
			for (int f = 0; f < z.length; f++) {
				
				int zonestrokes = (int)MathUtil.scale(z[f], min, max, 0, 5);
			
				for (int i = 0; i < zonestrokes; i++) {
					int colorindex = (int)MathUtil.scale(z[f % z.length], min, max, 0, palette.length-1);

					int x = RandomUtil.nextInt(t * zonewidth, (t+1) * zonewidth);
					int y = RandomUtil.nextInt(height - (f+1) * zoneheight, height - f * zoneheight);
					int w = RandomUtil.nextInt(5, zonewidth);
					int h = RandomUtil.nextInt(5, zoneheight);

					StraightBrushPainter p  = new StraightBrushPainter(x, y, w, h, ColorUtil.heatmap(z[f], min, max), RandomUtil.nextInt(0, 360));
					p.paintOn(image);
				}
			}
		}
		
		
		
		Logger.info("Oil ...");
		OilifyEffect oil = new OilifyEffect(image, 5, 15);
		image = oil.apply();
		
		Logger.info("Blur ...");
		BlurEffect blur = new BlurEffect(image, 1);
		image = blur.apply();
		
		return image;
	}

}
