package creim.art.artist.layerartists;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.QuadCurve2D;
import java.awt.image.BufferedImage;
import creim.analysis.audio.AudioAnalysisResult;
import creim.analysis.audio.AudioFileAnalyzer;
import creim.art.artist.Artist;
import creim.art.painter.CurveBrushPainter;
import org.apache.commons.math3.stat.StatUtils;



public class AvgFreqLineArtist extends Artist {

	private AudioAnalysisResult features;
	
	
	public AvgFreqLineArtist(int width, int height, AudioAnalysisResult features) {
		super(width, height);
		this.features = features;
	}

	
	@Override
	public BufferedImage paint() {
		Graphics2D g2 = (Graphics2D)image.getGraphics();
		
		AudioFileAnalyzer afa = new AudioFileAnalyzer();
		double[][] data = afa.highs(features.musicdata);
		double[] avgf = afa.meanFrequencies(data);
		double[] avg = new double[25];
		int parts = avgf.length/avg.length;
		for (int i = 0; i < avg.length; i++) {
			double[] tmp = new double[parts];
			for (int j = 0; j < parts; j++) {
				tmp[j] = avgf[i * parts + j];
			}
			double v = StatUtils.mean(tmp);
			avg[i] = v * 2;
		}
		
		double w = width / (double)data.length;
		double h = height / (double)data[0].length;
		
		double lx = 0;
		double ly = height - (avg[0] / (double)data[0].length) * height;
		
		for (int x = 1; x < avg.length; x += 2) {
			double  x1 = lx;
			double cx1 = ((x) / (double) (avg.length)) * width;
			double  x2 = ((x+1) / (double) (avg.length)) * width;
			
			double  y1 = ly;
			double cy1 = height - (avg[x] / (double)data[0].length) * height; 
			double  y2 = height - (avg[x+1] / (double)data[0].length) * height; 
			
			CurveBrushPainter brush = new CurveBrushPainter(15, Color.red);
			Shape shape = new QuadCurve2D.Double(x1, y1, cx1, cy1, x2, y2);
			brush.addShape(shape);
			brush.paintOn(image);
			
			lx = x2;
			ly = y2;
		}
		
		g2.dispose();
		
		return image;
	}
	
}
