package creim.graphics.effect;

import com.jhlabs.image.GaussianFilter;
import java.awt.Color;
import java.awt.image.BufferedImage;
import creim.Config;



public class BlurEffect extends ImageEffect {

	private int radius;
	private int threads;
	public static boolean gaussian = true;
	
	public BlurEffect(BufferedImage image, int radius) {
		super(image);
		this.radius = radius;
		this.threads = Config.EFFECT_THREADS;
	}
	
	
	public void applyInplace() {
		BufferedImage res = apply();
		for (int x = 0; x < image.getWidth(); x++) {
			for (int y = 0; y < image.getHeight(); y++) {
				image.setRGB(x, y, res.getRGB(x, y));
			}
		}
	}
	

	@Override
	public BufferedImage apply() {
		BufferedImage out = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_ARGB);
		if (gaussian) {
			gaussian(image, out);
		}
		else {
			int rs = (int)Math.ceil(radius * 2.57);
			int r  = radius;
			int w  = image.getWidth();
			int h  = image.getHeight();

			Thread[] pool = new Thread[threads];
			int len = w / threads;
			for (int i = 0; i < threads; i++) {
				int start = len * i;
				int end = Math.min(len * (i+1), w);
				pool[i] = new Thread(() -> {
					worker(image, out, start, end, w, h, r, rs);
				});
				pool[i].start();
			}
			for (int i = 0; i < threads; i++) {
				try { pool[i].join(); } catch (InterruptedException e) {}
			}
		}
		
		return out;
	}
	
	
	public void worker(BufferedImage in, BufferedImage out, int start, int end, int w, int h, int r, int rs) {
		for (int j = start; j < end; j++) {
			for (int i = 0; i < h; i++) {
				double aval = 0, rval = 0, gval = 0, bval = 0;
				double wsum = 0;
				
				for (int ix = j-rs; ix < j+rs+1; ix++) {
					for (int iy = i-rs; iy < i+rs+1; iy++) {
						int x = Math.min(w-1, Math.max(0, ix));
						int y = Math.min(h-1, Math.max(0, iy));
						int dsq = (ix-j)*(ix-j)+(iy-i)*(iy-i);
						double wght = Math.exp( -dsq / (2*r*r) ) / (Math.PI*2*r*r);
						
						int argb = in.getRGB(x, y);
						aval += ((argb >> 24) & 0xff) * wght; 
						rval += ((argb >> 16) & 0xff) * wght; 
						gval += ((argb >>  8) & 0xff) * wght; 
						bval += ((argb      ) & 0xff) * wght;
						wsum += wght;
					}
				}
				int na = (int)Math.round(aval / wsum);
				int nr = (int)Math.round(rval / wsum);
				int ng = (int)Math.round(gval / wsum);
				int nb = (int)Math.round(bval / wsum);
				
				int nargb = new Color(nr, ng, nb, na).getRGB();
				
				out.setRGB(j, i, nargb);
			}
		}
	}
	
	
	private void gaussian(BufferedImage in, BufferedImage out) {
		GaussianFilter filter = new GaussianFilter(radius);
		filter.filter(in, out);
	}
}
