package creim.graphics.effect;

import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.graphics.colors.ColorUtil;



public class CloudEffect extends ImageEffect {
	
	public static OpenSimplexNoise osn = new OpenSimplexNoise(RandomUtil.RNG.nextLong());
	
	private int alpha;
	
	public CloudEffect(BufferedImage image, int alpha) {
		super(image);
		this.alpha = alpha;
	}
	
	
	@Override
	public BufferedImage apply() {
		int width = image.getWidth();
		int height = image.getHeight();
		
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				double nx = x*10/(double)width - 0.5;
				double ny = y*10/(double)height - 0.5;
				
//				double n = noise(nx, ny);
				
				double n =   1.0 * noise(1.0 * nx, 1.0 * ny)
				         +   0.5 * noise(2.0 * nx, 2.0 * ny)
				         +  0.25 * noise(4.0 * nx, 4.0 * ny)
				         +  0.13 * noise(8.0 * nx, 8.0 * ny)
				         +  0.06 * noise(16.0 * nx, 16.0 * ny);
				
				n /= (1.0 + 0.5 + 0.25 + 0.13 + 0.06);
				
				int p = (int)MathUtil.scale(n, 0, 255);
				int c = (alpha << 24) | p << 16 | p << 8 | p;
				int o = image.getRGB(x, y);
				int m = ColorUtil.mixNorm(c, o);
				
				image.setRGB(x, y, m);
			}
		}
		return image;
	}
	
	public static double noise(double x, double y) {
		return osn.eval(x, y) / 2.0 + 0.5;
	}
	
	public static void reset() {
		osn = new OpenSimplexNoise(RandomUtil.RNG.nextLong());
	}
	

}
