package creim.graphics.effect;

import java.awt.image.BufferedImage;

public class OilifyEffect extends ImageEffect {
	
	private int radius;
	private double intensity;

	public OilifyEffect(BufferedImage image, int radius, double intensity) {
		super(image);
		this.radius = radius;
		this.intensity = intensity;
	}
	
	
	public void applyInplace() {
		BufferedImage res = apply();
		for (int x = 0; x < image.getWidth(); x++) {
			for (int y = 0; y < image.getHeight(); y++) {
				image.setRGB(x, y, res.getRGB(x, y));
			}
		}
	}
	

	@Override
	public BufferedImage apply() {
		BufferedImage out = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_ARGB);
		
		int width = image.getWidth();
		int height = image.getHeight();

		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				
				int[] intencityCount = new int[256];
				int[] sumr = new int[256];
				int[] sumg = new int[256];
				int[] sumb = new int[256];
				
				for (int xr = -radius; xr <= radius; xr++) {
					for (int yr = -radius; yr <= radius; yr++) {
						if (x+xr < 0 || x+xr >= width || y+yr < 0 || y+yr >= height)
							continue;
						
						int rgb = image.getRGB(x+xr, y+yr);
						
						int r = (rgb >>> 16) & 0xff;
						int g = (rgb >>>  8) & 0xff;
						int b = (rgb       ) & 0xff;

						// Find intensity of RGB value and apply intensity level.
						int currIntensity = (int)((((r + g + b) / 3.0) * intensity) / 255.0);
						if (currIntensity > 255) {
							currIntensity = 255;
						}
						int i = currIntensity;
						intencityCount[i]++;

						sumr[i] = sumr[i] + r;
						sumg[i] = sumg[i] + g;
						sumb[i] = sumb[i] + b;
					}
				}

				int max = 0;
				int maxidx = 0;
				for (int i = 0; i < 256; i++) {
					if (intencityCount[i] > max) {
						max = intencityCount[i];
						maxidx = i;
					}
				}

				int nr = sumr[maxidx] / max;
				int ng = sumg[maxidx] / max;
				int nb = sumb[maxidx] / max;
				
				int nrgb = (255 << 24) | (nr << 16) | (ng << 8) | (nb);
				out.setRGB(x, y, nrgb);
//				System.out.println("x: "+(x)+"  y: "+(y)+"   rgb: "+new Color(nrgb));
			}
		}

		return out;
	}

}
