package creim.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.imageio.ImageIO;
import javax.swing.JFrame;

public class ImageFrame extends JFrame {

	public ImageFrame(BufferedImage img) throws HeadlessException {
		this(img.getWidth(), img.getHeight(), img);
	}
	
	public ImageFrame(int w, int h, BufferedImage img) throws HeadlessException {
		this(w, h, img, null);
	}
	
	public ImageFrame(int w, int h, BufferedImage img, Color background) throws HeadlessException {
		super();
		super.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		ImagePanel panel = new ImagePanel(new Dimension(w, h));
		
		if (background != null) {
			super.setBackground(background);
			super.getContentPane().setBackground(background);
			panel.setBackground(background);
		}
		
		panel.showImage(img);
		super.addKeyListener(new KeyListener() {
			@Override
			public void keyTyped(KeyEvent e) {}

			@Override
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_S) {
					saveImage(img);
				}
			}

			@Override
			public void keyReleased(KeyEvent e) {}
		});
		super.add(panel);
		super.pack();
		super.setLocationRelativeTo(null);
	}
	
	
	public void saveImage(BufferedImage img) {
		try { 
			SimpleDateFormat format = new SimpleDateFormat("YYYYMMddHHmmss");
			File savefile = new File("save"+ format.format(new Date()) +"_"+ this.getTitle() +".png");
			ImageIO.write(img, "png", savefile);
			System.out.println("Saved: "+savefile);
		} catch (IOException ex) {}
	}
}

