package creim.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;



public class ImagePanel extends JPanel {

	private BufferedImage image;
	private int           imgw;
	private int           imgh;
	private Dimension     max;
	private Dimension     scaled;
	private Point         pos;
	private Point         mstart;
	private Point         mnow;
	private Point         save;
	private double        scale;
	private boolean       zoom;
	private boolean       move;
	
	public ImagePanel(Dimension max) {
		super();
		
		this.max = max;
		this.setPreferredSize(max);
		this.setBackground(Color.BLACK); 
		this.scaled = new Dimension(max);
		this.pos = new Point();
		this.addComponentListener(new ResizeListener());
		this.scale = 1.0;
		this.zoom = false;
		this.move = false;
		this.mnow = new Point();
		this.mstart = new Point();
		this.save = new Point();
		
		addMouseListener(new MouseListener() {
			@Override public void mouseClicked(MouseEvent e) {}
			@Override 
			public void mousePressed(MouseEvent e) {
				int b = e.getButton();				
				if (b == MouseEvent.BUTTON1) {
					mstart = e.getPoint();
					save.x = pos.x;
					save.y = pos.y;
					move = true;
				}
			}
			@Override 
			public void mouseReleased(MouseEvent e) {
				move = false;
				int b = e.getButton();
				if (b == MouseEvent.BUTTON3) {
					zoom = !zoom;
					scale = 1.0;
//					recalculate(imgw, imgh);
					
					repaint();
				}
			}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
		});
		
		addMouseMotionListener(new MouseMotionListener() {
			@Override
			public void mouseDragged(MouseEvent e) {
				if (!move) 
					return;
				mnow = e.getPoint();
				int dx = mnow.x - mstart.x;
				int dy = mnow.y - mstart.y;
				pos.x = save.x + (int)(dx);
				pos.y = save.y + (int)(dy);
				repaint();
			}
			@Override public void mouseMoved(MouseEvent e) {}
		});
		
		addMouseWheelListener((MouseWheelEvent e) -> {
			double scale0 = scale;
			if (e.getWheelRotation() < 0) {
				scale *= 1.1;
			} else {
				scale *= 0.9;
			}
			double ds = scale - scale0;
			Point m = e.getPoint();
			pos.x += (int)(-m.x * ds);
			pos.y += (int)(-m.y * ds);
			repaint();
		});
	}
	
	
	class ResizeListener extends ComponentAdapter {
		@Override
		public void componentResized(ComponentEvent e) {
			max.width = e.getComponent().getWidth();
			max.height = e.getComponent().getHeight();
			
//			recalculate(imgw, imgh);
		}
	}
	

	@Override
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		super.paint(g2);
		if (image == null) 
			return;
		
		g2.translate(pos.x, pos.y);
		g2.scale(scale, scale);
		g2.drawImage(image, 0, 0, imgw, imgh, null);
	}
	
	
	public void clear() {
		image = null;
	}

	
	public void showImage(BufferedImage img) {
		image = img;
		if (img == null)
			return;
		
		imgh = image.getHeight();
		imgw = image.getWidth();
		
		if (imgw < max.width && imgh < max.height) {
			pos.y = max.height / 2 - imgh / 2;
			pos.x = max.width / 2 - imgw / 2;
		} else {
//			recalculate(imgw, imgh);
		}
	}
	
	
	
	public void recalculate(int imgw, int imgh) {
		pos.y = max.height / 2 - imgh / 2;
		pos.x = max.width / 2 - imgw / 2;
		if (!zoom) {
			double ratiow = imgw / (double)max.width;
			double ratioh = imgh / (double)max.height;

			if (ratioh > ratiow) {
				scale = 1 / ratioh;
				pos.y = 0;
				pos.x = max.width / 2 - (int)(imgw * scale / 2);
			}
			else {
				scale = 1 / ratiow;
				pos.x = 0;
				pos.y = max.height / 2 - (int)(imgh * scale / 2);
			}
		}
	}
	
	
}
