package creim.oldea;


import eval.models.Crossover;
import eval.models.Genotype;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.CubicCurve2D;
import java.util.ArrayList;
import javalib.util.RandomUtil;
import creim.art.painter.*;
import creim.graphics.colors.ColorPalette;
import creim.util.Logger;
import org.apache.commons.collections4.list.TreeList;



public class ArtGenotype extends Genotype {
	
	public final Dimension imagedim;
	public TreeList<Painter> painters;
	public Color[] palette;
	public int margin;

	public ArtGenotype(Dimension imagedim, int margin, Color[] palette) {
		this.painters = new TreeList<>();
		this.imagedim = imagedim;
		this.palette = palette;
		this.margin = margin;
	}
	
	public ArtGenotype(TreeList<Painter> painters, Dimension imagedim, int margin, Color[] palette) {
		this.painters = painters;
		this.imagedim = imagedim;
		this.palette = palette;
		this.margin = margin;
	}


	@Override
	public Genotype copy() {
		TreeList<Painter> copies = new TreeList<>();
		Dimension imagedimcopy = new Dimension(imagedim.width, imagedim.height);
		Color[] palettecopy = new Color[palette.length];
		for (int i = 0; i < palettecopy.length; i++) {
			palettecopy[i] = new Color(palette[i].getRGB(), true);
		}
		
		for (Painter p : painters) {
			if (p instanceof CircleBrushPainter) {
				CircleBrushPainter orig = (CircleBrushPainter)p;
				CircleBrushPainter copy = new CircleBrushPainter();
				copy.brushcolor = new Color(orig.brushcolor.getRGB(), true);
				copy.brushwidth = orig.brushwidth;
				copy.x = orig.x;
				copy.y = orig.y;
				copy.w = orig.w;
				copy.h = orig.h;
				copies.add(copy);
			}
			else if (p instanceof CurveBrushPainter) {
				CurveBrushPainter orig = (CurveBrushPainter)p;
				CurveBrushPainter copy = new CurveBrushPainter();
				copy.brushwidth = orig.brushwidth;
				copy.brushcolor = new Color(orig.brushcolor.getRGB(), true);
				copy.shapes = new ArrayList<>();
				for (Shape shape : orig.shapes) {
					CubicCurve2D.Double s = (CubicCurve2D.Double)shape;
					CubicCurve2D.Double shapecopy = new CubicCurve2D.Double(s.x1, s.y1, s.ctrlx1, s.ctrly1, s.ctrlx2, s.ctrly2, s.x2, s.y2);
					copy.shapes.add(shapecopy);
				}
				copies.add(copy);
			}
			else if (p instanceof StraightBrushPainter) {
				StraightBrushPainter orig = (StraightBrushPainter)p;
				StraightBrushPainter copy = new StraightBrushPainter();
				copy.brushcolor = new Color(orig.brushcolor.getRGB(), true);
				copy.x = orig.x;
				copy.y = orig.y;
				copy.brushwidth = orig.brushwidth;
				copy.length = orig.length;
				copy.rotationdegrees = orig.rotationdegrees;
				copies.add(copy);
			}
			else if (p instanceof OvalPainter) {
				OvalPainter orig = (OvalPainter)p;
				OvalPainter copy = new OvalPainter();
				copy.color = new Color(orig.color.getRGB(), true);
				copy.outlineColor = new Color(orig.outlineColor.getRGB(), true);
				copy.outlineThickness = orig.outlineThickness;
				copy.x = orig.x;
				copy.y = orig.y;
				copy.w = orig.w;
				copy.h = orig.h;
				copies.add(copy);
			}
			else if (p instanceof RectanglePainter) {
				RectanglePainter orig = (RectanglePainter)p;
				RectanglePainter copy = new RectanglePainter();
				copy.color = new Color(orig.color.getRGB(), true);
				copy.outlineColor = new Color(orig.outlineColor.getRGB(), true);
				copy.outlineThickness = orig.outlineThickness;
				copy.x = orig.x;
				copy.y = orig.y;
				copy.w = orig.w;
				copy.h = orig.h;
				copies.add(copy);
			}
			else if (p instanceof PolygonPainter) {
				PolygonPainter orig = (PolygonPainter)p;
				PolygonPainter copy = new PolygonPainter();
				copy.color = new Color(orig.color.getRGB(), true);
				copy.outlineColor = new Color(orig.outlineColor.getRGB(), true);
				copy.outlineThickness = orig.outlineThickness;
				copy.points = new ArrayList<>();
				for (Point op : orig.points) {
					copy.points.add(new Point(op));
				}
				copies.add(copy);
			} 
			else {
				Logger.error("ArtGenotype copy error. Unkown painter: "+p.getClass());
				throw new RuntimeException("Error, genotype copy fail");
			}
		}
		
		
		return new ArtGenotype(copies, imagedimcopy, margin, palettecopy);
	}

	
	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		ArtGenotype other = (ArtGenotype)g;
		ArtGenotype newgeno = (ArtGenotype)this.copy();
		
		int end = RandomUtil.nextInt(this.painters.size());
		for (int i = end; i < newgeno.painters.size() - end; i++) {
			newgeno.painters.remove(end);
		}
		
		int start = RandomUtil.nextInt(other.painters.size());
		for (int i = start; i < other.painters.size(); i++) {
			newgeno.painters.add(other.painters.get(i));
		}
		
		return newgeno;
	}

	
	@Override
	public void mutate(double prob) {
		if (RandomUtil.randBool(prob)) {
			float f = RandomUtil.RNG.nextFloat();
			if (f <= 0.7) {
				int index = RandomUtil.nextInt(painters.size());
				mutatePainter(painters.get(index));
			}
			else if (f <= 0.85) {
				if (RandomUtil.randBool(0.5)) 
					removePainter();
				else
					addPainter();
			}
			else {
				try {
					palette = ColorPalette.getRandomPalette();
				} catch (Exception e) {
					palette = ColorPalette.generateRandomPastelPalette();
				}
			}
		}
	}
	
	private void removePainter() {
		int index = RandomUtil.nextInt(painters.size());
		painters.remove(index);
	}
	
	private void addPainter() {
		Painter painter = Painter.randomPainter(imagedim, margin, palette);
		int index = RandomUtil.nextInt(painters.size());
		painters.add(index, painter);
	}
	
	private void mutatePainter(Painter painter) {
		painter.mutate(imagedim, palette, margin);
	}
}
