package master;

import creim.analysis.audio.spotifyapi.SpotifyAPI;
import creim.analysis.audio.spotifyapi.SpotifyAudioFeatures;
import creim.art.artist.ea.ArtistEA;
import creim.art.artist.ea.ArtistGenotype;
import creim.art.artist.ea.ArtistPhenotype;
import creim.art.artist.ea.MappingArtist;
import creim.art.artist.ea.fitnesscalculators.ComboDistanceFitnessCalculator;
import creim.art.artist.ea.fitnesscalculators.FitnessTarget;
import creim.business.MusicToImage;
import creim.util.Logger;
import eval.ea.EAResults;
import eval.ea.OnChangedEvent;
import eval.util.Config;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import javax.imageio.ImageIO;

/**
 *
 * @author Viktor
 */
public class NtnuMaster {
	
	
	public static final String ROOTPREFIX = "save";
//	public static final String experiment = "user_with_seed";
	public static final String experiment = "user_combo";
//	public static final String[] trackIDs = {"spotify:track:5ChkMS8OtdzJeqyybCc9R5", "spotify:track:2iNRom9yU8lY7SEw6JCiBA"};
	public static final String[] trackIDs = {"spotify:track:5f2CgxUSiRtuEMcY6mmAW4", "spotify:track:6qTJtlUrLTrZLUuS81SPI7", "spotify:track:2iNRom9yU8lY7SEw6JCiBA", "spotify:track:5pD9x23lsoQr464mlmaKNJ", "spotify:track:2L8ZiTLLejiY9O7qhhoday", "spotify:track:5ChkMS8OtdzJeqyybCc9R5"};
	public static String[] names = null;
	public static Config eacfg;
	
	
	public static void main(String[] args) throws Exception {
		initTheme();
		SimpleDateFormat format = new SimpleDateFormat("YYYY-MM-dd_HH-mm-ss");
		File rootdir = new File("output/"+experiment, ROOTPREFIX+"_"+ format.format(new Date()));
		Logger.log = true;
		Logger.logtofile = true;
		Logger.setOutputDir(rootdir);
		
		eacfg = new Config("ArtistEA.properties");
		Logger.debug(eacfg);
		
		EARun(rootdir);
//		generateRandom(rootdir, 5);
	}
	
	
	public static void EARun(File rootdir) throws Exception {
//		FitnessTarget.loadHardCoded4();
		Logger.debug("FitnessTarget: "+FitnessTarget.genotype);
		Logger.debug("Description: "+ComboDistanceFitnessCalculator.targetgenotype);
		Logger.debug("Description: "+ComboDistanceFitnessCalculator.expectedmap);
		
		
		ArrayList<ArtistGenotype> seeds = new ArrayList<>();
		File seedroot = new File("output/fitness_target");
		deserializeAll(seedroot, seeds);
		
		Dimension imagedim = new Dimension(eacfg.getInt("image_width"), eacfg.getInt("image_height"));
		ArtistGenotype.mutationPressure = eacfg.getDouble("mutation_pressure");
		SpotifyAPI sapi = new SpotifyAPI();
		
		names = new String[trackIDs.length];
		SpotifyAudioFeatures[] features = new SpotifyAudioFeatures[trackIDs.length];
		for (int i = 0; i < trackIDs.length; i++) {
			Logger.notify("SpotifyAPI getting features for "+ trackIDs[i]);
			String name = sapi.getTrackName(trackIDs[i]);
			names[i] = name;
			features[i] = sapi.getAudioFeatureObject(trackIDs[i]);
			Logger.notify("SpotifyAPI done with "+ trackIDs[i] +" -> "+ name);
		}
		
		
		ArtistEA ea = new ArtistEA(eacfg, imagedim, features);
		
//		ea.seedPopulation(seeds);
//		new Thread(() -> {
//			boolean abort = JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(null, "Abort?");
//			if (abort)
//				ea.abort();
//		}).start();
		
		Logger.debug("EA started");
		Logger.log = false;
		
		ea.addListener((e) -> {
			if (e == OnChangedEvent.NEW_BEST_FITNESS) {
				ArtistPhenotype pheno = (ArtistPhenotype)ea.getWinner();
				ArtistGenotype geno = (ArtistGenotype)pheno.getGenotype();
				try {
					storeGenotype(rootdir, geno, "genotypes/genotype_"+pheno.getFitness()+".csv");
				} catch (IOException ex) {
					Logger.error(ex.getMessage());
				} 
				ArtistGenotype.mutationPressure = pheno.getFitness() / 10.0;
				System.out.println("Mutation pressure: "+ArtistGenotype.mutationPressure);
			}
		});
	
		ea.evolve();
		
		Logger.log = true;
		Logger.debug("EA done");
		
		ArtistPhenotype pheno = (ArtistPhenotype)ea.getWinner();
		ArtistGenotype geno = (ArtistGenotype)pheno.getGenotype();
		Logger.debug(pheno);
		
		ArrayList<MusicToImage> mtms = new ArrayList<>();
		for (int i = 0; i < features.length; i++) {
			MappingArtist ma = new MappingArtist(imagedim, features[i], geno, true);
			BufferedImage image = ma.paint();
			String name = sapi.getTrackName(trackIDs[i]);
			MusicToImage mtm = new MusicToImage(name, image);
			mtms.add(mtm);
		}
		
		storeEARun(rootdir, ea);
		storeGenotype(rootdir, geno);
		storeImageResults(rootdir, mtms);
	}
	
	
	public static void generateRandom(File rootdir, int count) throws IOException {
		
		Dimension imagedim = new Dimension(eacfg.getInt("image_width"), eacfg.getInt("image_height"));
		SpotifyAPI sapi = new SpotifyAPI();
		
		SpotifyAudioFeatures[] features = new SpotifyAudioFeatures[trackIDs.length];
		for (int i = 0; i < trackIDs.length; i++) {
			features[i] = sapi.getAudioFeatureObject(trackIDs[i]);
		}
		
		for (int i = 0; i < count; i++) {
			File setfile = new File(rootdir, "set_"+i);
			
			ArtistGenotype randomgeno = ArtistGenotype.generateRandom();
			System.out.println(randomgeno);
			
			ArrayList<MusicToImage> mtms = new ArrayList<>();
			for (int j = 0; j < features.length; j++) {
				MappingArtist ma = new MappingArtist(imagedim, features[j], randomgeno, true);
				BufferedImage image = ma.paint();
				String name = sapi.getTrackName(trackIDs[j]);
				MusicToImage mtm = new MusicToImage(name, image);
				mtms.add(mtm);
			}

			storeGenotype(setfile, randomgeno);
			storeImageResults(setfile, mtms);
		}
		
	}
	
	
	public static void storeEARun(File rootdir, ArtistEA ea) throws IOException {
		rootdir.mkdirs();
		EAResults eares = ea.getResults();
		File earun = new File(rootdir, "earun.csv");
		eares.writeExcelCompatibleTable(earun.toString(), true);
		System.out.println("Stored "+earun);
	}
	
	public static void storeGenotype(File rootdir, ArtistGenotype geno) throws IOException {
		storeGenotype(rootdir, geno, "genotype.csv");
	}
	
	public static void storeGenotype(File rootdir, ArtistGenotype geno, String filename) throws IOException {
		File genofile = new File(rootdir, filename);
		genofile.getParentFile().mkdirs();
		geno.serialize(genofile);
		System.out.println("Stored "+genofile);
	}
	
	public static void storeImageResults(File rootdir, ArrayList<MusicToImage> mtms) throws IOException {
		rootdir.mkdirs();
		for (MusicToImage mtm : mtms) {
			File file = new File(rootdir, mtm.name+".png");
			ImageIO.write(mtm.image, "PNG", file);
			System.out.println("Stored "+file);
		}
	}
	
	
	public static void deserializeAll(File rootdir, ArrayList<ArtistGenotype> seeds) {
		if (rootdir == null || !rootdir.exists()) {
			Logger.error("Missing root dir");
			return;
		}
			
			for (File root : rootdir.listFiles()) {
				File genofile = new File(root, "genotype.csv");
				if (!genofile.exists()) 
					continue;
				try {
					ArtistGenotype seed = new ArtistGenotype();
					Logger.debug("Deserializing: "+genofile);
					seed.deserialize(genofile);
					seeds.add(seed);
				} catch (Exception e) {
					Logger.error("Deserializing: "+genofile);
				}
			}
	}
	
	
	private static void initTheme() {
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Windows".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (Exception ex) {}
	}
}
